package com.jianshui.invoiceall.utils;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.jianshui.common.core.domain.entity.Companyservice;
import com.jianshui.common.utils.spring.SpringUtils;
import com.jianshui.system.domain.CompanyserviceProp;
import com.jianshui.system.service.ICompanyservicePropService;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 金财数科
 * @author dxk
 * @date 2022年12月09日 16:14
 */
public class ApiHttp {

    private static final Logger logger = LoggerFactory.getLogger(ApiHttp.class);


    /**
     * 请求接口封装
     * @throws Exception
     */
    public static JSONObject request(String serviceId,String apiName, Object reqBody, Companyservice companyservices) throws Exception{

        if (BeanUtil.isEmpty(companyservices)) {
            throw new Exception("未配置系统服务，请联系管理员！");
        }

        ICompanyservicePropService propService = SpringUtils.getBean(ICompanyservicePropService.class);

        //获取token配置参数
        CompanyserviceProp propAuthKey = propService.selectPropByKey(companyservices.getCompanyid(), "jcsk_invoice_all_appkey");
        String propAuthKeyS = getValue(propAuthKey.getValue());
        CompanyserviceProp propAuthSecret = propService.selectPropByKey(companyservices.getCompanyid(), "jcsk_invoice_all_appsecret");
        String propAuthSecretS = getValue(propAuthSecret.getValue());

        if (StrUtil.isEmpty(propAuthKeyS) || StrUtil.isEmpty(propAuthSecretS)) {
            throw new Exception("未配置系统app key/app secret，请联系管理员！！!");
        }

        JSONObject ywJson = JSONUtil.createObj();//业务报文
        ywJson.putOpt("appkey",propAuthKey)
                .putOpt("async",false)
                .putOpt("serviceId",serviceId)
                .putOpt("dqbm","")
                .putOpt("nsrsbh","")
                .putOpt("bsrysfzjhm","")
                .putOpt("jsonData",JSONUtil.parse(reqBody));

        logger.info("金四请求接口：{},原始报文：{},企业税号:{}",apiName,reqBody,companyservices.getSellertax());
        String codeData = AESUtils.encrypt(JSONUtil.parseObj(ywJson).toString(), propAuthSecretS);
        long timestamp = System.currentTimeMillis();
        String sig = DigestUtils.md5Hex(propAuthKeyS + propAuthSecretS + codeData + timestamp);
        JSONObject requestBody = JSONUtil.createObj();

        logger.info("金四请求codeData:{},sig:{}",codeData,sig);
        requestBody.putOpt("appkey",propAuthKeyS)
                .putOpt("version","1")
                .putOpt("encryptStr", codeData)
                .putOpt("timestamp",timestamp)
                .putOpt("sign",sig);

        String result= HttpUtil.post( apiName, requestBody.toString());
        logger.info("接口返回:"+result);

        // TODO-kk: 这个到时候带根据具体的返回调整下

        String data = JSONUtil.parseObj(result).getStr("data");
        logger.info("data解密后:"+AESUtils.decrypt(data,propAuthSecretS));
        String res = AESUtils.decrypt(data,propAuthSecretS);

        return  JSONUtil.parseObj(res);
    }


    /**
     * Object toString
     *
     * @param obje
     * @return
     */
    public static String getValue(Object obje) {
        return obje != null ? obje.toString() : "";
    }



}
